#include "Entity.h"

#include <cassert>

using namespace DirectX;


Entity::Entity(int hitpoints)
	: mVelocity(0.0f, 0.0f, 0.0f)
	, mHitpoints(hitpoints)
{
}

void Entity::setVelocity(DirectX::XMFLOAT3 velocity)
{
	mVelocity = velocity;
}

void Entity::setVelocity(float vx, float vy, float zy)
{
	mVelocity.x = vx;
	mVelocity.y = vy;
}

DirectX::FXMVECTOR Entity::getVelocity() const
{
	return XMLoadFloat3(&mVelocity);
}

void Entity::accelerate(DirectX::XMFLOAT3 velocity)
{
	mVelocity.x += velocity.x;
	mVelocity.y += velocity.y;
	mVelocity.z += velocity.z;
}

void Entity::accelerate(float vx, float vy, float vz)
{
	mVelocity.x += vx;
	mVelocity.y += vy;
	mVelocity.z += vz;
}

int Entity::getHitpoints() const
{
	return mHitpoints;
}

void Entity::repair(int points)
{
	assert(points > 0);

	mHitpoints += points;
}

void Entity::damage(int points)
{
	assert(points > 0);

	mHitpoints -= points;
}

void Entity::destroy()
{
	mHitpoints = 0;
}

bool Entity::isDestroyed() const
{
	return mHitpoints <= 0;
}

void Entity::move(DirectX::XMFLOAT3 translate)
{
	mTrasforms.mTranslate.x += translate.x;
	mTrasforms.mTranslate.y += translate.y;
	mTrasforms.mTranslate.z += translate.z;
}

void Entity::updateCurrent(float dt, CommandQueue& commands)
{	
	XMFLOAT3 velocity;
	velocity.x = mVelocity.x * dt;
	velocity.y = mVelocity.y * dt;
	velocity.z = mVelocity.z * dt;
	move(velocity);
}