#include "ModelHolder.h"

using namespace DirectX;

ModelHolder::ModelHolder(ID3D11Device * d3dDevice, DirectX::IEffectFactory & fxFactory)
	: md3dDevice{ d3dDevice }
	, mfxFactory{ &fxFactory }
{
}

void ModelHolder::load(Models::ID id, const std::wstring& filename)
{
	// Create and load DirectX::Model
	std::unique_ptr<DirectX::Model> model = DirectX::Model::CreateFromCMO(md3dDevice, filename.c_str(), *mfxFactory);
	if (model.get() == nullptr) {
		std::wstring wstr;
		wstr = L"ModelHolder::load - Failed to load " + filename;
		std::string str(wstr.begin(), wstr.end());
		throw std::runtime_error(str);
	}
	// If loading successful, insert DirectX::Model to map
	insertResource(id, std::move(model));
}

DirectX::Model & ModelHolder::get(Models::ID id)
{
	auto found = mResourceMap.find(id);
	assert(found != mResourceMap.end());

	return *found->second;
}

const DirectX::Model & ModelHolder::get(Models::ID id) const
{
	auto found = mResourceMap.find(id);
	assert(found != mResourceMap.end());

	return *found->second;
}

void ModelHolder::insertResource(Models::ID id, std::unique_ptr<DirectX::Model> model)
{
	// Insert and check success
	auto inserted = mResourceMap.insert(std::make_pair(id, std::move(model)));
	assert(inserted.second);
}
