#include "Obstacle.h"

#include "Category.h"
#include "CommandQueue.h"

#include "DataTables.h"

namespace
{
	const std::vector<ObstacleData> Table = initializeObstacleData();
}


using namespace std::chrono;

Obstacle::Obstacle(Type type, ModelHolder & models)
	: Entity(1)
	, mIsMarkedForRemoval(false)
	, mType(type)
{
	// TODO - uwzglednic model z typu
	mModel = &models.get(Table[type].modelID);
	mBoundingOrientedBox.Center = mModel->meshes[0]->boundingBox.Center;
	mBoundingOrientedBox.Extents = mModel->meshes[0]->boundingBox.Extents;

	mLiveTime = milliseconds(1000 / 24);
	mCreateTime = high_resolution_clock::now();
}

unsigned int Obstacle::getCategory() const
{
	return Category::Obstacle;
}

BoundingOrientedBox Obstacle::getBoundingRect() const
{
	BoundingOrientedBox transformedAABB;
	XMMATRIX transforms = getWorld() * getWorldTransform();
	mBoundingOrientedBox.Transform(transformedAABB, transforms);
	return transformedAABB;
}

bool Obstacle::isMarkedForRemoval() const
{
	return mIsMarkedForRemoval || isDestroyed();
}

void Obstacle::apply(Snake & player) const
{
	Table[mType].action(player);
	OutputDebugStringA("Obstacle::apply(Snake & player)\n");
}

void Obstacle::updateCurrent(float dt, CommandQueue & commands)
{
	if (mCreateTime + mLiveTime < high_resolution_clock::now())
	{
		mIsMarkedForRemoval = true;
	}
	//mIsMarkedForRemoval = true;


	/*static float sx = 0.0f;
	static float sy = 0.0f;
	static float sz = 0.0f;

	float speed = 0.02f;
	if (sx < XM_2PI)
		sx += speed;
	else if (sy < XM_2PI)
		sy += speed;
	else if (sz < XM_2PI)
		sz += speed;

	if (sx < XM_2PI && sy >= XM_2PI && sz >= XM_2PI) {
		sx -= XM_2PI;
		sy -= XM_2PI;
		sz -= XM_2PI;
	}

	mTrasforms.mRotate.x = sx;
	mTrasforms.mRotate.y = sy;
	mTrasforms.mRotate.z = sz;*/
}

DirectX::FXMMATRIX& getCameraView(Camera* camera, const DirectX::XMFLOAT4X4& world)
{
	auto pos = camera->GetPosition();
	camera->SetPosition(world._41, world._42, world._43);
	camera->UpdateViewMatrix();
	auto view = camera->View();
	camera->SetPosition(pos);
	return view;
}

void Obstacle::drawCurrent(ID3D11DeviceContext * dc, const Camera & camera, const CommonStates & states) const
{
	XMMATRIX world = getWorld() * getWorldTransform();
	mModel->Draw(dc, states, world, camera.View(), camera.Proj());
	

	g_debugEffect->SetView(camera.View());
	g_debugEffect->SetProjection(camera.Proj());

	dc->OMSetBlendState(states.Opaque(), nullptr, 0xFFFFFFFF);
	dc->OMSetDepthStencilState(states.DepthNone(), 0);
	dc->RSSetState(states.CullNone());

	g_debugEffect->Apply(dc);

	dc->IASetInputLayout(g_debugLayout.get());

	g_debugBatch->Begin();
	world = XMMatrixScaling(0.5f, 0.5f, 0.5f) * world;
	//DX::DrawCube(g_debugBatch.get(), world, Colors::GreenYellow); // BoundingFrustum

	g_debugBatch->End();
}

void Obstacle::drawBoundingRect(ID3D11DeviceContext * deviceContext, const Camera & camera, const DirectX::CommonStates & states) const
{
	g_debugEffect->SetView(camera.View());
	g_debugEffect->SetProjection(camera.Proj());

	deviceContext->OMSetBlendState(states.Opaque(), nullptr, 0xFFFFFFFF);
	deviceContext->OMSetDepthStencilState(states.DepthNone(), 0);
	deviceContext->RSSetState(states.CullNone());

	g_debugEffect->Apply(deviceContext);

	deviceContext->IASetInputLayout(g_debugLayout.get());

	g_debugBatch->Begin();
	auto aaa = getBoundingRect();
	//BoundingBox out;
	//aaa.Transform(out, XMLoadFloat4x4(&mWorld));
	DX::Draw(g_debugBatch.get(), aaa, Colors::Cyan); // BoundingFrustum

	g_debugBatch->End();
}

