#include "Pickup.h"

#include "Category.h"
#include "CommandQueue.h"

#include "DataTables.h"

namespace
{
	const std::vector<PickupData> Table = initializePickupData();
}


using namespace std::chrono;

Pickup::Pickup(Type type, ModelHolder & models)
	: Entity(1)
	, mIsMarkedForRemoval(false)
	, mType(type)
{
	// TODO - uwzglednic model z typu
	mModel = &models.get(Table[type].model);
	mBoundingOrientedBox.Center = mModel->meshes[0]->boundingBox.Center;
	mBoundingOrientedBox.Extents = mModel->meshes[0]->boundingBox.Extents;

	mLiveTime = seconds(20);
	mCreateTime = high_resolution_clock::now();
}

unsigned int Pickup::getCategory() const
{
	return Category::Pickup;
}

BoundingOrientedBox Pickup::getBoundingRect() const
{
	// TODO
	BoundingOrientedBox transformedAABB;
	XMMATRIX transforms = getWorld() * getWorldTransform();
	mBoundingOrientedBox.Transform(transformedAABB, transforms);
	return transformedAABB;
}

bool Pickup::isMarkedForRemoval() const
{
	return mIsMarkedForRemoval || isDestroyed();
}

void Pickup::apply(Snake & player) const
{
	Table[mType].action(player);
	OutputDebugStringA("Pickup::apply(Snake & player)\n");
}

void Pickup::updateCurrent(float dt, CommandQueue & commands)
{
	if (mCreateTime + mLiveTime < high_resolution_clock::now())
	{
		mIsMarkedForRemoval = true;
	}

	mTrasforms.mRotate.z += XM_PI * dt;
	if (mTrasforms.mRotate.z > XM_2PI)
		mTrasforms.mRotate.z -= XM_2PI;
}

void Pickup::drawCurrent(ID3D11DeviceContext * dc, const Camera & camera, const CommonStates & states) const
{
	XMMATRIX wt = this->getWorldTransform();
	XMMATRIX world = mTrasforms.World() * wt;
	mModel->Draw(dc, states, world, camera.View(), camera.Proj());
}

void Pickup::drawBoundingRect(ID3D11DeviceContext * deviceContext, const Camera & camera, const DirectX::CommonStates & states) const
{
	g_debugEffect->SetView(camera.View());
	g_debugEffect->SetProjection(camera.Proj());

	deviceContext->OMSetBlendState(states.Opaque(), nullptr, 0xFFFFFFFF);
	deviceContext->OMSetDepthStencilState(states.DepthNone(), 0);
	deviceContext->RSSetState(states.CullNone());

	g_debugEffect->Apply(deviceContext);

	deviceContext->IASetInputLayout(g_debugLayout.get());

	g_debugBatch->Begin();

	DX::Draw(g_debugBatch.get(), getBoundingRect(), Colors::Cyan); // BoundingFrustum

	g_debugBatch->End();
}