#include "Player.h"

#include <iostream>

#include "SceneNode.h"
#include "Snake.h"
#include "Command.h"

using namespace DirectX;

Player::Player()
	: tracker()
	, isSpaceEvent(false)
	, isREvent(false)
{
	keyboard = std::make_unique<Keyboard>();
}


Player::~Player()
{
}

void Player::handleEvent(CommandQueue & commands)
{
	if (tracker.pressed.P)
	{
		Command output;
		output.category = Category::PlayerSnake;
		output.action = [](SceneNode& s, float)
		{
			std::cout << s.getTransform().mTranslate.x << ","
				<< s.getTransform().mTranslate.y << ","
				<< s.getTransform().mTranslate.z << "\n";
		};
		commands.push(output);
	}

	isSpaceEvent = false;
	if (tracker.pressed.Space)
	{
		isSpaceEvent = true;
	}

	isREvent = false;
	if (tracker.pressed.R)
	{
		isREvent = true;
	}
}

void Player::handleRealtimeInput(CommandQueue & commands)
{
	auto state = keyboard->GetState();

	if (state.Up) {
		// rb cos gdy wciniety klawisz Up
		// np.
		Command resetPosition;
		resetPosition.category = Category::PlayerSnake;
		resetPosition.action = derivedAction<Snake>([](Snake& a, float dt) { a.accelerate(DirectX::XMFLOAT3(0.0f, 0.0f, 0.0f)); });
		commands.push(resetPosition);
	}
}

bool Player::isLeft()
{
	return keyboard->GetState().Left;
}

bool Player::isRight()
{
	return keyboard->GetState().Right;
}

bool Player::isUp()
{
	return keyboard->GetState().Up;
}

bool Player::isDown()
{
	return keyboard->GetState().Down;
}

LRESULT Player::WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_ACTIVATEAPP:
	{
		DirectX::Keyboard::ProcessMessage(message, wParam, lParam);
		auto state = DirectX::Keyboard::Get().GetState();
		tracker.Reset();
		tracker.Update(state);
		break;
	}

	case WM_KEYDOWN:
	case WM_SYSKEYDOWN:
	case WM_KEYUP:
	case WM_SYSKEYUP:
	{
		DirectX::Keyboard::ProcessMessage(message, wParam, lParam);
		auto state = DirectX::Keyboard::Get().GetState();
		tracker.Update(state);
		break;

	}
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}
