#include "Snake.h"

#include <Keyboard.h>

using namespace std;
using namespace std::chrono;
using namespace DirectX;

Snake::Snake(ModelHolder& models)
	: Entity(10)
	, mIsGrowing(false)
	, mIsMarkedForRemoval(false)
	, mPoints(0)
	, mLivies(3)
	, mStepTime(std::chrono::milliseconds(300))
	, mDir(Direction::Up)
	, mNextDir(Direction::Up)
{
	// TODO - nie wiem dlaczego wywyuje mytody const zamiast non-const
	mHeadModel = &models.get(Models::SnakeHead);
	mBodyModel = &models.get(Models::SnakeBody);
	mTailModel = &models.get(Models::SnakeBody);

	// make head as a part of snake body
	mModelInstances.push_back(std::make_unique<ModelInstance>(mHeadModel, Transforms()));

	// save head size
	XMStoreFloat3(&mHeadSize, XMLoadFloat3(&mHeadModel->meshes[0]->boundingBox.Extents) * 2.0f);
	XMStoreFloat3(&mBodySize, XMLoadFloat3(&mBodyModel->meshes[0]->boundingBox.Extents) * 2.0f);
}

Snake::~Snake()
{
}

unsigned int Snake::getCategory() const
{
	return Category::PlayerSnake;
}

DirectX::BoundingOrientedBox Snake::getBoundingRect() const
{
	XMMATRIX world;

	auto wt = getWorldTransform();
	auto& head = (*mModelInstances.begin());
	{
		auto& bbox = head->model->meshes[0]->boundingBox;
		BoundingOrientedBox obbox;
		obbox.Center = bbox.Center;
		obbox.Extents = bbox.Extents;
		obbox.Extents.z *= 10;

		world = head->transforms.World() * wt;
		BoundingOrientedBox obbox_trans;
		obbox.Transform(obbox_trans, world);
		return obbox_trans;
		//DX::Draw(g_debugBatch.get(), obbox_trans, Colors::Red);
		//DX::DrawCube(g_debugBatch.get(), world, Colors::Yellow);
		//instance->model->Draw(deviceContext, states, world, view, proj, false, nullptr);
	}
}

bool Snake::isMarkedForRemoval() const
{
	return mIsMarkedForRemoval;
}

void Snake::updateCurrent(float dt, CommandQueue & commands)
{ 
	// Entity has been destroyed
	if (isDestroyed() || mLivies <= 0)
	{
		return;
	}

	static auto lastStepTime = high_resolution_clock::now();
	auto currTime = high_resolution_clock::now();
	if (currTime - lastStepTime >= mStepTime)
	{
		this->move();
		lastStepTime += mStepTime;
	}

	//Entity::updateCurrent(dt, commands);
}

void Snake::drawCurrent(ID3D11DeviceContext* dc, const Camera& camera, const CommonStates& states) const
{
	XMMATRIX world;
	XMMATRIX view = camera.View();
	XMMATRIX proj = camera.Proj();

	auto wt = getWorldTransform();
	for (auto& instance : mModelInstances)
	{
		world = instance->transforms.World() * wt;
		instance->model->Draw(dc, states, world, view, proj, false, nullptr);

		//instance->model->Draw(dc, states, world * XMMatrixTranslation(10.0f, 0.0f, 0.0f), view, proj, false, nullptr);

		//instance->model->Draw(dc, states, world * XMMatrixTranslation(-10.0f, 0.0f, 0.0f), view, proj, false, nullptr);

		//instance->model->Draw(dc, states, world * XMMatrixTranslation(0.0f, 20.0f, 0.0f), view, proj, false, nullptr);
	}
}

void Snake::handleInput()
{
	static Keyboard::KeyboardStateTracker tracker;
	auto state = DirectX::Keyboard::Get().GetState();
	tracker.Update(state);

	Turn turn;
	if (tracker.pressed.Left)
		turn = Turn::Right;
	else if (tracker.pressed.Right)
		turn = Turn::Left;
	else
		return;
	// sad

	if (turn == Turn::Left)
	{
		int dir = (static_cast<int>(mDir) + 1) % static_cast<int>(Direction::DirCount);
		mNextDir = static_cast<Direction>(dir);
	}
	else if (turn == Turn::Right)
	{
		int dir = (static_cast<int>(mDir) - 1) % static_cast<int>(Direction::DirCount);
		if (dir < 0)
			dir = static_cast<int>(Direction::DirCount) - 1;

		mNextDir = static_cast<Direction>(dir);
	}

	std::stringstream strs;
	strs << "Snake::handleInput(DIR = " << (int)mDir << ") mNextDir = " << (int)mNextDir << std::endl;
	OutputDebugStringA(strs.str().c_str());
}

void Snake::looseLive()
{
	--mLivies;

	if (mLivies <= 0)
		destroy();
}

void Snake::addLive()
{
	++mLivies;
}

void Snake::loosePoints(unsigned int points)
{
	mPoints -= points;
	if (mPoints < 0)
		mPoints = 0;
}

void Snake::addPoints(unsigned int points)
{
	mPoints += points;
}

void Snake::grow()
{
	mIsGrowing = true;
}

void Snake::resetPosition()
{
	auto head = std::move(mModelInstances.front());
	mModelInstances.clear();
	head->transforms = Transforms();
	mModelInstances.push_back(std::move(head));
}

void Snake::move()
{
	//XMMATRIX headTranslation;
	//XMMATRIX headRotation;
	float dx = 0.0f;
	float dy = 0.0f;
	float r = 0.0f;
	switch (mNextDir)
	{
	case Direction::Left:
		//OutputDebugStringA("Snake::move(LEFT)\n");
		//headRotation = XMMatrixRotationX(XMConvertToRadians(90.0f));
		//headTranslation = XMMatrixTranslation(mHeadSize.x, 0.0f, 0.0f);
		dy = mHeadSize.x;
		r = -XM_PIDIV2;
		OutputDebugStringA("Snake::move(+x)\n");
		break;
	case Direction::Right:
		//OutputDebugStringA("Snake::move(Right)\n");
		//headRotation = XMMatrixRotationX(XMConvertToRadians(-90.0f));
		//headTranslation = XMMatrixTranslation(-mHeadSize.x, 0.0f, 0.0f);
		dy = -mHeadSize.x;
		r = XM_PIDIV2;
		OutputDebugStringA("Snake::move(-x)\n");
		break;
	case Direction::Up:
		//OutputDebugStringA("Snake::move(UP)\n");
		//headRotation = XMMatrixRotationZ(XMConvertToRadians(0.0f));
		//headTranslation = XMMatrixTranslation(0.0f, -mHeadSize.z, 0.0f);
		dx = -mHeadSize.z;
		r = 0.0;
		OutputDebugStringA("Snake::move(-z)\n");
		break;
	case Direction::Down:
		//OutputDebugStringA("Snake::move(DOWN)\n");
		//headRotation = XMMatrixRotationZ(XMConvertToRadians(180.0f));
		//headTranslation = XMMatrixTranslation(0.0f, mHeadSize.z, 0.0f);
		dx = mHeadSize.z;
		r = -XM_PI;
		OutputDebugStringA("Snake::move(+z)\n");
		break;
	default:
		throw std::logic_error("Unsupported Direction type!");
	}

	mDir = mNextDir;

	// new head transforms
	ModelInstance* head = (*mModelInstances.begin()).get();
	Transforms t = head->transforms;
	t.mTranslate.x += dx;
	t.mTranslate.y += dy;
	t.mRotate.z = r;

	if (mModelInstances.size() == 1 && !mIsGrowing)
		// set head in new position
		//XMStoreFloat4x4(&(*mModelInstances.begin())->world, newHeadPosition);
		head->transforms = t;
	else
	{
		// head becomes body
		(*mModelInstances.begin())->model = mBodyModel;

		// add new head
		mModelInstances.push_front(std::make_unique<ModelInstance>(mHeadModel, t));

		if (mIsGrowing)
			// do not remove tail
			mIsGrowing = false;
		else if (mModelInstances.size() > 2)
			// remove tail
			mModelInstances.pop_back();

		mIsGrowing = false;
	}

	OutputDebugStringA("Instances");
	OutputDebugStringA(std::to_string(mModelInstances.size()).c_str());
	OutputDebugStringA("\n");

}

void Snake::drawBoundingRect(ID3D11DeviceContext * deviceContext, const Camera & camera, const DirectX::CommonStates & states) const
{
	g_debugEffect->SetView(camera.View());
	g_debugEffect->SetProjection(camera.Proj());

	deviceContext->OMSetBlendState(states.Opaque(), nullptr, 0xFFFFFFFF);
	deviceContext->OMSetDepthStencilState(states.DepthNone(), 0);
	deviceContext->RSSetState(states.CullNone());

	g_debugEffect->Apply(deviceContext);

	deviceContext->IASetInputLayout(g_debugLayout.get());

	g_debugBatch->Begin();
	XMMATRIX world;
	XMMATRIX view = camera.View();
	XMMATRIX proj = camera.Proj();

	auto wt = getWorldTransform();
	for (auto& instance : mModelInstances)
	{
		auto& bbox = instance->model->meshes[0]->boundingBox;
		BoundingOrientedBox obbox;
		obbox.Center = bbox.Center;
		obbox.Extents = bbox.Extents;
		obbox.Extents.z *= 10;

		world = instance->transforms.World() * wt;
		BoundingOrientedBox obbox_trans;
		obbox.Transform(obbox_trans, world);
		DX::Draw(g_debugBatch.get(), obbox_trans, Colors::Cyan);
		//DX::DrawCube(g_debugBatch.get(), world, Colors::Green);
		//instance->model->Draw(deviceContext, states, world, view, proj, false, nullptr);
	}

	g_debugBatch->End();
}