#pragma once

#include <d3dUtil.h>
#include "Camera.h"
#include "Entity.h"

#include <list>
#include <memory>
#include <chrono>

#include <Model.h>

#include "ModelHolder.h"
#include "ModelInstance.h"
#include "Player.h"

class Snake : public Entity
{
public:
	typedef XMFLOAT3 Size;

	enum class Direction {
		Left = 0,
		Down = 1,
		Right = 2,
		Up = 3,
		DirCount
	};

	enum class Turn
	{
		Left, Right
	};


public:
	Snake(ModelHolder& models);
	~Snake();

	unsigned int getCategory() const override;
	DirectX::BoundingOrientedBox getBoundingRect() const override;
	bool isMarkedForRemoval() const override;

	void handleInput();

	void looseLive();
	void addLive();
	void loosePoints(unsigned int points);
	void addPoints(unsigned int points);
	int  getLives() { return mLivies; }
	int  getPoints() { return mPoints; }

	void grow();
	void resetPosition();
	
private:
	void updateCurrent(float dt, CommandQueue& commands) override;
	void drawCurrent(ID3D11DeviceContext* dc, const Camera& camera, const CommonStates& states) const override;
	void drawBoundingRect(ID3D11DeviceContext * deviceContext, const Camera & camera, const DirectX::CommonStates & states) const;

private:
	void move();


private:
	bool mIsMarkedForRemoval;
	
	bool mIsGrowing;
	int mPoints;
	int mLivies;
	Direction mDir, mNextDir;
	const std::chrono::milliseconds mStepTime;

	Size mHeadSize;
	Size mBodySize;

	DirectX::Model* mHeadModel;
	DirectX::Model* mBodyModel;
	DirectX::Model* mTailModel;

	std::list<std::unique_ptr<ModelInstance>> mModelInstances;

	ID3D11DeviceContext* md3dImmediateContext;
};

