#pragma once

#include "d3dApp.h"
#include "Camera.h"

#include <Effects.h>
#include <CommonStates.h>
#include <GeometricPrimitive.h>
#include <PrimitiveBatch.h>
#include <Model.h>

#include <SimpleMath.h>
#include "World.h"
#include "Snake.h"
#include "Player.h"

class SnakeApp : public D3DApp
{
public:
	SnakeApp(HINSTANCE hInstance);
	~SnakeApp();

	bool Init();
	void OnResize();
	void UpdateScene(float dt);
	void DrawScene();
	LRESULT MsgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

	void OnMouseDown(WPARAM btnState, int x, int y);
	void OnMouseUp(WPARAM btnState, int x, int y);
	void OnMouseMove(WPARAM btnState, int x, int y);

	CommonStates& getCommonStates() const;
	DirectX::IEffectFactory* getEffectFactory() const;

private:
	void loadModels();

private:
	std::unique_ptr<Context> mContext;

	std::unique_ptr<World> mWorld;
	Player mPlayer;

	std::unique_ptr<ModelHolder> mModels;
	std::unique_ptr<DirectX::SpriteBatch> m_spriteBatch;
	std::unique_ptr<DirectX::CommonStates> states;
	std::unique_ptr<DirectX::IEffectFactory> m_fxFactory;

	POINT mLastMousePos;
};