#include "TextNode.h"


TextNode::TextNode(SpriteFont* font)
	: SceneNode(Category::Text)
	, m_font(font)
	, mIsMarkedForRemoval(false)
{
}

TextNode::TextNode(SpriteFont* font, int x, int y)
	: SceneNode(Category::Text)
	, m_font(font)
	, mPosition(x, y)
	, mIsMarkedForRemoval(false)
{
}

TextNode::~TextNode()
{
	m_font = nullptr;
}

unsigned int TextNode::getCategory() const
{
	return Category::Text;
}

bool TextNode::isMarkedForRemoval() const
{
	return mIsMarkedForRemoval;
}

void TextNode::setPosition(int x, int y)
{
	mPosition.x = x;
	mPosition.y = y;
}

void TextNode::setOrigin(int x, int y)
{
	mOrigin.x = x;
	mOrigin.y = y;
}

void TextNode::setText(std::wstring text)
{
	mText = text;
}

std::wstring TextNode::getText()
{
	return mText;
}

void TextNode::updateCurrent(float dt, CommandQueue & commands)
{
	if (isDestroyed())
		mIsMarkedForRemoval = true;
}

void TextNode::drawCurrent(ID3D11DeviceContext * dc, const Camera & camera, const CommonStates & states) const
{
	g_spriteBatch->Begin();

	m_font->DrawString(g_spriteBatch.get(), mText.c_str(),
		XMLoadFloat2(&mPosition), Colors::White, 0.f, XMLoadFloat2(&mOrigin));

	g_spriteBatch->End();
}
