#pragma once

#include "d3dUtil.h"

#include <thread>
#include <mutex>

#include <opencv2\highgui.hpp>

#include <MarkerAR\CameraCalibration.h>

class ViewController;

// TO DO
// napisac reopen video stream jezeli nastapi rozlaczenie lub koniec video

class VideoSource
{
public:
	explicit VideoSource(ID3D11Device* device, ID3D11DeviceContext* immediateContext, int camera_index);
	explicit VideoSource(ID3D11Device* device, ID3D11DeviceContext* immediateContext, std::string source);
	~VideoSource();

	void Play();
	void Stop();
	
	bool IsPlaying();
	bool IsNotRetrivedFrame();

	const cv::Mat GetAsMat() const;
	ID3D11Texture2D* GetAsD3D11Texture2D();
	ID3D11ShaderResourceView* LastFrameSRV();

	CameraCalibration getCalibration()
	{
		// Returns calibratoin data for iPad 2
		// Todo: Add parameters for the rest
		return CameraCalibration(624.860291f * (640.0f / 352.0f), 624.860291f * (480.0f / 288.0f), 640 * 0.5f, 480 * 0.5f);
	}

	cv::Size getFrameSize()
	{
		cv::Size cameraSize;

		if (mVideoCap.isOpened())
		{
			cameraSize.width = static_cast<int>(mVideoCap.get(CV_CAP_PROP_FRAME_WIDTH));
			cameraSize.height = static_cast<int>(mVideoCap.get(CV_CAP_PROP_FRAME_HEIGHT));
		}
		else {
			std::cerr << "BLAD nie ma jeszcze rozmiaru!" << std::endl;
			return cv::Size(0, 0);
			mVideoCap.open(0);
			cameraSize.width = static_cast<int>(mVideoCap.get(CV_CAP_PROP_FRAME_WIDTH));
			cameraSize.height = static_cast<int>(mVideoCap.get(CV_CAP_PROP_FRAME_HEIGHT));
		}
		
		return cameraSize;
	}

public:
	ViewController* mDelegate;

private:
	VideoSource(ID3D11Device * device, ID3D11DeviceContext* immediateContext);

	void OpenVideo();

	void ReleasePlayVideoThread();

	void PlayVideoThread();

	std::unique_ptr<ID3D11Texture2D> CreateTexture(int width, int height);
	// get media data on DX surface for further processing
	void get_surface(ID3D11Texture2D** ppSurface);

private:
	ID3D11Device* md3dDevice;
	ID3D11DeviceContext* md3dImmediateContext;

	std::thread* mVideoPlayThread;
	mutable std::mutex mFrameMutex;
	bool mIsRunningThread;
	bool mTryReopen;
	mutable bool mIsNotRetrivedFrame;

	cv::VideoCapture mVideoCap;
	cv::Mat mCurrFrame;
	std::unique_ptr<ID3D11Texture2D> m_pSurfaceRGBA;
	std::unique_ptr<ID3D11ShaderResourceView> mLastSRV;
	mutable std::mutex mLastSRVMutex;
	bool mIsActualSRV;

	bool mIsPlaying;

	int mCamIndex;
	std::string mSource;
	int mFPS;
};

