#include "World.h"

#include "SnakeApp.h"
#include <MarkerAR\Marker.h>

#include <opencv2\calib3d.hpp>

using namespace std::chrono;

void ceterOrigin(TextNode* node, SpriteFont* font)
{
	auto tm = font->MeasureString(node->getText().c_str());
	XMFLOAT2 msf;
	XMStoreFloat2(&msf, tm);
	node->setOrigin(msf.x / 2, msf.y / 2);
}

World::World(Context context)
	: mCamera()
	, mSceneGraph()
	, mSceneLayers()
	, mWorldBounds(-10, -10, 20, 20)
	, mSpawnPosition(0.0f, 0.0f, 0.0f)
	, mPlayerSnake(nullptr)
	, mRandNewAppleTime(2)
	, mCameraCalibration("../cali.yml")
	//, mCameraCalibration(624.860291f * (1920.0f / 352.0f), 624.860291f * (1080.0f / 288.0f), 1920.0f * 0.5f, 1080.0f * 0.5f)
	, mMarkerDetector(mCameraCalibration)
	, mContext(context)
	, isPlaying(false)
	, restart(false)
{
	buildScene();

	XMMATRIX I = XMMatrixIdentity();
	XMStoreFloat4x4(&mBoardWord, I);

	// Prepare the view
	mCamera.SetLens(0.25f*MathHelper::Pi, mContext.app->AspectRatio(), 1.0f, 1000.0f);
	mCamera.SetPosition(10.0f, 0.0f, 0.0f);
	//mCamera.Pitch(XMConvertToRadians(45.0f));
	mCamera.LookAt(XMFLOAT3(0.0f, 0.0f, 0.0f), XMFLOAT3(0.0f, 0.0f, -1.0f), XMFLOAT3(0.0f, 1.0f, 0.0f));

	m_font = std::make_unique<SpriteFont>(mContext.d3dDevice, L"Fonts/myfile.spritefont");
	mPlayerText = std::make_unique<TextNode>(m_font.get());
	mPauseText = std::make_unique<TextNode>(m_font.get());
	mPauseText->setText(L"Nacisnij SPACJA by kontynuowac");
	mPauseText->setPosition(mContext.app->getClientRect().right / 2, mContext.app->getClientRect().bottom / 2);
	auto sad = m_font->MeasureString(L"asdasd asd a");
	ceterOrigin(mPauseText.get(), m_font.get());
	mRestartText = std::make_unique<TextNode>(m_font.get());
	mRestartText->setText(L"Nacisnij R by rozpoczac ponownie");
	mRestartText->setPosition(mContext.app->getClientRect().right / 2, mContext.app->getClientRect().bottom / 2);
	ceterOrigin(mRestartText.get(), m_font.get());

	//mVideoSource = std::make_unique<VideoSource>(mContext.d3dDevice, mContext.d3dImmediateContext, 0);
	mVideoSource = std::make_unique<VideoSource>(mContext.d3dDevice, mContext.d3dImmediateContext, "../../Video/test.mp4");
	//mVideoSource = std::make_unique<VideoSource>(mContext.d3dDevice, mContext.d3dImmediateContext, "../../Video/static.mp4");
	//mVideoSource = std::make_unique<VideoSource>(mContext.d3dDevice, mContext.d3dImmediateContext, "../../Video/static_low.mp4");
	mVideoSource->Play();
}

World::~World()
{
	//mVideoSource->Stop();
}

void World::Update(float dt)
{
	//mCamera.LookAt(XMFLOAT3(0, 0, 0), XMFLOAT3(0, 0, -1), XMFLOAT3(0, 1, 0));

	processMarkerDetector();


	auto t1 = high_resolution_clock::now();
	mPlayerSnake->handleInput();
	mPlayerSnake->setVelocity(0.0f, 0.0f, 0.0f);

	// Setup commands to destroy entities, and guide missiles
	//destroyEntitiesOutsideView();
	//guideMissiles();

	//generate collections
	randApple();

	// Forward commands to scene graph, adapt velocity (scrolling, diagonal correction)
	while (!mCommandQueue.isEmpty())
		mSceneGraph.onCommand(mCommandQueue.pop(), dt);
	adaptPlayerVelocity();

	// Collision detection and response (may destroy entities)
	handleCollisions();

	// Remove all destroyed entities, create new ones
	mSceneGraph.removeWrecks();
	//spawnEnemies();


	// Regular update step, adapt position (correct if outside view)
	mSceneGraph.Update(dt, mCommandQueue);
	adaptPlayerPosition();

	std::wstringstream str;
	str << "Zycia:  " << mPlayerSnake->getLives() << std::endl;
	str << "Punkty: " << mPlayerSnake->getPoints();
	mPlayerText->setText(str.str().c_str());
	mPlayerText->setPosition(10, 10);

	//XMFLOAT3 mRight(1.0f, 0.0f, 0.0f);
	//XMFLOAT3 mUp(0.0f, 1.0f, 0.0f);
	//XMFLOAT3 mLook(0.0f, 0.0f, 1.0f);
	//mCamera.LookAt(mRight, mLook, mUp);
	mCamera.UpdateViewMatrix();

	auto t2 = high_resolution_clock::now();
	auto d = duration_cast<milliseconds>(t2 - t1).count();
	std::stringstream strs;
	strs << "World::Update() time = " << d << "ms." << std::endl;
	//OutputDebugStringA(strs.str().c_str());
}

void World::Draw()
{
	auto t1 = high_resolution_clock::now();

	if (mVideoSource->IsPlaying())
	{
		mContext.spriteBatch->Begin(SpriteSortMode::SpriteSortMode_Immediate, nullptr, nullptr, mContext.states->DepthNone());
		mContext.spriteBatch->Draw(mVideoSource->LastFrameSRV(), mContext.app->getClientRect());
		mContext.spriteBatch->End();
	}


	mPlayerText->Draw(mContext.d3dImmediateContext, mCamera, *mContext.states);

	//if (isPlaying == false && playerDied == false)
	//	mPauseText->Draw(mContext.d3dImmediateContext, mCamera, *mContext.states);

	if (playerDied == true)
		mRestartText->Draw(mContext.d3dImmediateContext, mCamera, *mContext.states);

	mSceneGraph.Draw(mContext.d3dImmediateContext, mCamera, *mContext.states);
	auto t2 = high_resolution_clock::now();
	auto d = duration_cast<milliseconds>(t2 - t1).count();
	std::stringstream strs;
	strs << "World::Draw() time = " << d << "ms." << std::endl;
	//OutputDebugStringA(strs.str().c_str());
}

CommandQueue & World::getCommandQueue()
{
	return mCommandQueue;
}

Camera & World::getCamera()
{
	return mCamera;
}

void World::buildScene()
{
	// Initialize the different layers
	for (std::size_t i = 0; i < LayerCount; ++i)
	{
		SceneNode::Ptr layer(new SceneNode());
		mSceneLayers[i] = layer.get();

		mSceneGraph.attachChild(std::move(layer));
	}

	// Add player's snake
	std::unique_ptr<Snake> snake(std::make_unique<Snake>(*mContext.models));
	mPlayerSnake = snake.get();
	mPlayerSnake->setPosition(mSpawnPosition);
	mSceneLayers[ARBoard]->attachChild(std::move(snake));

	// add obstacle at random pos
	std::unique_ptr<Obstacle> obstacle(std::make_unique<Obstacle>(Obstacle::Palm, *mContext.models));
	obstacle->setPosition(10.0f, 0.0f, 15.0f);
	mSceneLayers[ARBoard]->attachChild(std::move(obstacle));

	// add obstacle at random pos
	//std::unique_ptr<Pickup> applesda(std::make_unique<Pickup>(Pickup::Apple, mModels));
	//applesda->setPosition(12.0f, 0.0f, 17.0f);
	//mSceneLayers[Air]->attachChild(std::move(applesda));
}

void World::adaptPlayerPosition()
{
	// TODO - w przyszlosci
}

void World::adaptPlayerVelocity()
{
	 // TODO - w przyszoci
}

bool matchesCategories(SceneNode::Pair& colliders, Category::Type type1, Category::Type type2)
{
	unsigned int category1 = colliders.first->getCategory();
	unsigned int category2 = colliders.second->getCategory();

	// Make sure first pair entry has category type1 and second has type2
	if (type1 & category1 && type2 & category2)
		return true;
	else if (type1 & category2 && type2 & category1)
	{
		std::swap(colliders.first, colliders.second);
		return true;
	}
	else
		return false;
}

void World::handleCollisions()
{
	std::set<SceneNode::Pair> collisionPairs;
	mSceneGraph.checkSceneCollision(mSceneGraph, collisionPairs);

	for (SceneNode::Pair pair : collisionPairs)
	{
		if (matchesCategories(pair, Category::PlayerSnake, Category::Pickup))
		{
			auto& player = static_cast<Snake&>(*pair.first);
			auto& pickup = static_cast<Pickup&>(*pair.second);

			// Apply pickup effect to player, destroy projectile
			pickup.apply(player);
			pickup.destroy();
		}
		else if (matchesCategories(pair, Category::PlayerSnake, Category::Obstacle))
		{
			auto& player = static_cast<Snake&>(*pair.first);
			auto& obstacle = static_cast<Pickup&>(*pair.second);

			// Apply projectile damage to aircraft, destroy projectile
			player.looseLive();
			player.resetPosition();
		}
	}
}

void World::processMarkerDetector()
{
	if (!mVideoSource->IsNotRetrivedFrame())
		return;

	measureTime("GetAsMat()");
	cv::Mat frame = mVideoSource->GetAsMat();
	//logTime();

	static auto time = high_resolution_clock::now();
	static int frame_counter = 0;
	frame_counter++;

	measureTime("processFrame(frame)");
	mMarkerDetector.processFrame(frame);
	//logTime();

	measureTime("rest");
	// draw object on detected 
	auto transformations = mMarkerDetector.getTransformations();
	auto markers = mMarkerDetector.getMarkers();

	cv::Mat outPerst;
	mCameraCalibration.loadProjectionMatrix(true, outPerst);
	//outPerst = outPerst.t();
	float sum = 0.0f;
	for (int i = 0; i < 4; ++i) {
		for (int j = 0; j < 4; ++j) {
			//mCamera.mProjAR.m[i][j] = outPerst.at<double>(i, j);
			//sum += x;
		}
	}
	double fovx, fovy, focalLength, aspectRatio; cv::Point2d principalPt;
	cv::calibrationMatrixValues(mCameraCalibration.getIntrinsic(), cv::Size(1920, 1080), 0.0, 0.0, fovx, fovy, focalLength, principalPt, aspectRatio);

	//mCamera.SetLens(fovy, aspectRatio, 1.0f, 1000.0f);
	
	matchTransformationsToObjects(markers);
}

DirectX::FXMMATRIX World::Matrix44ToXMMatrix(Matrix44 & mat)
{
	XMFLOAT4X4 world;
	for (int i = 0; i < 4; ++i) {
		for (int j = 0; j < 4; j++) {
			world(i, j) = mat.mat[i][j];
		}
	}

	return XMLoadFloat4x4(&world);
}

void World::matchTransformationsToObjects(std::vector<Marker>& markers)
{
	if (markers.size() == 0)
		return;

	XMMATRIX rotationX = XMMatrixRotationX(XMConvertToRadians(180));
	XMMATRIX rotationY = XMMatrixRotationY(XMConvertToRadians(0));
	XMMATRIX rotationZ = XMMatrixRotationZ(XMConvertToRadians(180));
	XMMATRIX rotationX90 = XMMatrixRotationX(XMConvertToRadians(90));
	XMMATRIX rotationY90 = XMMatrixRotationY(XMConvertToRadians(90));
	XMMATRIX rotationZ90 = XMMatrixRotationZ(XMConvertToRadians(90));
	XMMATRIX rotationY180 = XMMatrixRotationY(XMConvertToRadians(0));
	XMMATRIX scale = XMMatrixScaling(-1.0f, -1.0f, 1.0f);

	for (size_t i = 0; i < markers.size() || i < 1; ++i)
	{	
		XMFLOAT4X4 fw;
		for (int k = 0; k < 4; ++k) {
			for (int j = 0; j < 4; ++j) {
				fw(k, j) = markers[i].transformation.mModelViewMatrix.at<double>(k, j);
			}
		}

		XMMATRIX world = Matrix44ToXMMatrix(markers[i].transformation.getMat44());

		world = XMLoadFloat4x4(&fw);// * XMMatrixScaling(1.0f, 1.0f, 1.0f);
		//world = XMMatrixIdentity();
		if (markers[i].id == 576) // set AR Layer world matrix
		{
			mSceneLayers[Layer::ARBoard]->mWorld = fw;
			/*mSceneLayers[Layer::ARBoard]->mTrasforms.mRotate.x = -markers[i].transformation.r_v().data[0];
			mSceneLayers[Layer::ARBoard]->mTrasforms.mRotate.y = -markers[i].transformation.r_v().data[1];
			mSceneLayers[Layer::ARBoard]->mTrasforms.mRotate.z = -markers[i].transformation.r_v().data[2];

			mSceneLayers[Layer::ARBoard]->mTrasforms.mTranslate.x = -markers[i].transformation.t().data[0];
			mSceneLayers[Layer::ARBoard]->mTrasforms.mTranslate.y = -markers[i].transformation.t().data[1];
			mSceneLayers[Layer::ARBoard]->mTrasforms.mTranslate.z = -markers[i].transformation.t().data[2];*/
		}
		else
		{
			std::unique_ptr<Obstacle> obstacle(new Obstacle(Obstacle::Type::Palm, *mContext.models));
			/*obstacle->mTrasforms.mRotate.x = markers[i].transformation.r_v().data[0];
			obstacle->mTrasforms.mRotate.y = markers[i].transformation.r_v().data[1];
			obstacle->mTrasforms.mRotate.z = markers[i].transformation.r_v().data[2];

			obstacle->mTrasforms.mTranslate.x = -markers[i].transformation.t().data[0];
			obstacle->mTrasforms.mTranslate.y = -markers[i].transformation.t().data[1];
			obstacle->mTrasforms.mTranslate.z = -markers[i].transformation.t().data[2];*/
			XMStoreFloat4x4(&obstacle->mWorld, world);
			XMStoreFloat4x4(&mSceneLayers[Layer::AR]->mWorld, XMMatrixIdentity());
			mSceneLayers[Layer::AR]->attachChild(std::move(obstacle));
		}
	}
}

std::vector<XMMATRIX>& World::getAllObjectsWorlds()
{
	std::vector<XMMATRIX> worlds;

	worlds.push_back(mSceneLayers[ARBoard]->getWorld());
	for (auto& sceneNode : mSceneLayers[ARBoard]->mChildren)
	{
		worlds.push_back(sceneNode->getWorld());
	}

	return worlds;
}

void World::randApple()
{
	static auto lastRandTime = high_resolution_clock::now();
	if (mRandNewAppleTime > high_resolution_clock::now() - lastRandTime)
		return;

	float x = static_cast<float>(std::rand() % static_cast<int>(mWorldBounds.w)) + mWorldBounds.x;
	float y = static_cast<float>(std::rand() % static_cast<int>(mWorldBounds.h)) + mWorldBounds.y; 
	float z = 0.0f;

	Pickup::Type type = static_cast<Pickup::Type>(std::rand() % Pickup::Type::TypeCount);
	std::unique_ptr<Pickup> pickup(new Pickup(type, *mContext.models));
	//mSceneGraph[Air]->a
	// TODO uwzglednic w tym przesuniecie mBoardMatrix
	//pickup->setPosition(x, y, z);
	pickup->mTrasforms.mTranslate.x = x;
	pickup->mTrasforms.mTranslate.y = y;
	pickup->mTrasforms.mTranslate.z = z;
	mSceneLayers[ARBoard]->attachChild(std::move(pickup));

	lastRandTime = high_resolution_clock::now();
}
