#pragma once

#include <SpriteBatch.h>
#include <SpriteFont.h>

#include "SceneNode.h"
#include "CommandQueue.h"
#include "Command.h"
#include "ModelHolder.h"
#include "Camera.h"
#include "Snake.h"
#include "Pickup.h"
#include "Obstacle.h"
#include "TextNode.h"

#include <array>
#include <queue>
#include <chrono>

#include "Common\d3dUtil.h"

#include <MarkerAR\MarkerDetector.h>
#include <MarkerAR\CameraCalibration.h>
#include "VideoSource.h"

class SnakeApp;

struct Context
{
	Context(SnakeApp& app, ID3D11Device* d3dDevice, ID3D11DeviceContext* d3dImmediateContext,
		ModelHolder& models, Player& player,
		DirectX::SpriteBatch& spriteBatch, DirectX::CommonStates& states)
		: app(&app)
		, d3dDevice(d3dDevice)
		, d3dImmediateContext(d3dImmediateContext)
		, models(&models)
		, player(&player)
		, spriteBatch(&spriteBatch)
		, states(&states)
	{}

	SnakeApp* app;

	ID3D11Device* d3dDevice;
	ID3D11DeviceContext* d3dImmediateContext;

	ModelHolder* models;
	Player* player;

	DirectX::SpriteBatch* spriteBatch;
	DirectX::CommonStates* states;
};

class World
{
public:
	World(Context context);
	~World();

	void Update(float dt);
	void Draw();

	CommandQueue& getCommandQueue();
	Camera& getCamera();

private:
	void buildScene();
	void adaptPlayerPosition();
	void adaptPlayerVelocity();
	void handleCollisions();
	void processMarkerDetector();
	DirectX::FXMMATRIX Matrix44ToXMMatrix(Matrix44& mat);
	void matchTransformationsToObjects(std::vector<Marker>& markers);
	std::vector<XMMATRIX>& getAllObjectsWorlds();
	void randApple();

private:
	enum Layer
	{
		Background,
		ARBoard,
		AR,
		LayerCount
	};

private:
	Camera mCamera;

	SceneNode mSceneGraph;
	std::array<SceneNode*, LayerCount> mSceneLayers;
	CommandQueue mCommandQueue;

	FloatRect mWorldBounds;
	DirectX::XMFLOAT3 mSpawnPosition;
	DirectX::XMFLOAT4X4 mBoardWord;
	Snake* mPlayerSnake;

	std::chrono::seconds mRandNewAppleTime;

	CameraCalibration mCameraCalibration;
	MarkerDetector mMarkerDetector;
	std::unique_ptr<VideoSource> mVideoSource;

	std::unique_ptr<DirectX::SpriteFont> m_font;
	std::unique_ptr<TextNode> mPlayerText;
	std::unique_ptr<TextNode> mPauseText;
	std::unique_ptr<TextNode> mRestartText;
	bool isPlaying;
	bool restart;
	bool playerDied;
	Context mContext;
};

